@@ -1,11 +1,13 @@
 package ldap
 
 import (
+	"context"
 	"crypto/tls"
 	"crypto/x509"
 	"fmt"
 	"io/ioutil"
 	"log"
+	"time"
 )
 
 // This example demonstrates how to bind a connection to an ldap user
@@ -49,6 +51,72 @@ func ExampleConn_Search() {
 	}
 }
 
+// This example demonstrates how to search asynchronously
+func ExampleConn_SearchAsync() {
+	l, err := DialURL(fmt.Sprintf("%s:%d", "ldap.example.com", 389))
+	if err != nil {
+		log.Fatal(err)
+	}
+	defer l.Close()
+
+	searchRequest := NewSearchRequest(
+		"dc=example,dc=com", // The base dn to search
+		ScopeWholeSubtree, NeverDerefAliases, 0, 0, false,
+		"(&(objectClass=organizationalPerson))", // The filter to apply
+		[]string{"dn", "cn"},                    // A list attributes to retrieve
+		nil,
+	)
+
+	ctx, cancel := context.WithCancel(context.Background())
+	defer cancel()
+
+	r := l.SearchAsync(ctx, searchRequest, 64)
+	for r.Next() {
+		entry := r.Entry()
+		fmt.Printf("%s has DN %s\n", entry.GetAttributeValue("cn"), entry.DN)
+	}
+	if err := r.Err(); err != nil {
+		log.Fatal(err)
+	}
+}
+
+// This example demonstrates how to do syncrepl (persistent search)
+func ExampleConn_Syncrepl() {
+	l, err := DialURL(fmt.Sprintf("%s:%d", "ldap.example.com", 389))
+	if err != nil {
+		log.Fatal(err)
+	}
+	defer l.Close()
+
+	searchRequest := NewSearchRequest(
+		"dc=example,dc=com", // The base dn to search
+		ScopeWholeSubtree, NeverDerefAliases, 0, 0, false,
+		"(&(objectClass=organizationalPerson))", // The filter to apply
+		[]string{"dn", "cn"},                    // A list attributes to retrieve
+		nil,
+	)
+
+	ctx, cancel := context.WithCancel(context.Background())
+	defer cancel()
+
+	mode := SyncRequestModeRefreshAndPersist
+	var cookie []byte = nil
+	r := l.Syncrepl(ctx, searchRequest, 64, mode, cookie, false)
+	for r.Next() {
+		entry := r.Entry()
+		if entry != nil {
+			fmt.Printf("%s has DN %s\n", entry.GetAttributeValue("cn"), entry.DN)
+		}
+		controls := r.Controls()
+		if len(controls) != 0 {
+			fmt.Printf("%s", controls)
+		}
+	}
+	if err := r.Err(); err != nil {
+		log.Fatal(err)
+	}
+}
+
 // This example demonstrates how to start a TLS connection
 func ExampleConn_StartTLS() {
 	l, err := DialURL("ldap://ldap.example.com:389")
@@ -344,11 +412,108 @@ func ExampleControlPaging_manualPaging() {
 	}
 }
 
+// This example demonstrates how to use DirSync to manually execute a
+// DirSync search request
+func ExampleConn_DirSync() {
+	conn, err := Dial("tcp", "ad.example.org:389")
+	if err != nil {
+		log.Fatalf("Failed to connect: %s\n", err)
+	}
+	defer conn.Close()
+
+	_, err = conn.SimpleBind(&SimpleBindRequest{
+		Username: "cn=Some User,ou=people,dc=example,dc=org",
+		Password: "MySecretPass",
+	})
+	if err != nil {
+		log.Fatalf("failed to bind: %s", err)
+	}
+
+	req := &SearchRequest{
+		BaseDN:     `DC=example,DC=org`,
+		Filter:     `(&(objectClass=person)(!(objectClass=computer)))`,
+		Attributes: []string{"*"},
+		Scope:      ScopeWholeSubtree,
+	}
+	// This is the initial sync with all entries matching the filter
+	doMore := true
+	var cookie []byte
+	for doMore {
+		res, err := conn.DirSync(req, DirSyncObjectSecurity, 1000, cookie)
+		if err != nil {
+			log.Fatalf("failed to search: %s", err)
+		}
+		for _, entry := range res.Entries {
+			entry.Print()
+		}
+		ctrl := FindControl(res.Controls, ControlTypeDirSync)
+		if ctrl == nil || ctrl.(*ControlDirSync).Flags == 0 {
+			doMore = false
+		}
+		cookie = ctrl.(*ControlDirSync).Cookie
+	}
+	// We're done with the initial sync. Now pull every 15 seconds for the
+	// updated entries - note that you get just the changes, not a full entry.
+	for {
+		res, err := conn.DirSync(req, DirSyncObjectSecurity, 1000, cookie)
+		if err != nil {
+			log.Fatalf("failed to search: %s", err)
+		}
+		for _, entry := range res.Entries {
+			entry.Print()
+		}
+		time.Sleep(15 * time.Second)
+	}
+}
+
+// This example demonstrates how to use DirSync search asynchronously
+func ExampleConn_DirSyncAsync() {
+	conn, err := Dial("tcp", "ad.example.org:389")
+	if err != nil {
+		log.Fatalf("Failed to connect: %s\n", err)
+	}
+	defer conn.Close()
+
+	_, err = conn.SimpleBind(&SimpleBindRequest{
+		Username: "cn=Some User,ou=people,dc=example,dc=org",
+		Password: "MySecretPass",
+	})
+	if err != nil {
+		log.Fatalf("failed to bind: %s", err)
+	}
+
+	req := &SearchRequest{
+		BaseDN:     `DC=example,DC=org`,
+		Filter:     `(&(objectClass=person)(!(objectClass=computer)))`,
+		Attributes: []string{"*"},
+		Scope:      ScopeWholeSubtree,
+	}
+
+	ctx, cancel := context.WithCancel(context.Background())
+	defer cancel()
+
+	var cookie []byte = nil
+	r := conn.DirSyncAsync(ctx, req, 64, DirSyncObjectSecurity, 1000, cookie)
+	for r.Next() {
+		entry := r.Entry()
+		if entry != nil {
+			entry.Print()
+		}
+		controls := r.Controls()
+		if len(controls) != 0 {
+			fmt.Printf("%s", controls)
+		}
+	}
+	if err := r.Err(); err != nil {
+		log.Fatal(err)
+	}
+}
+
 // This example demonstrates how to use EXTERNAL SASL with TLS client certificates.
 func ExampleConn_ExternalBind() {
-	var ldapCert = "/path/to/cert.pem"
-	var ldapKey = "/path/to/key.pem"
-	var ldapCAchain = "/path/to/ca_chain.pem"
+	ldapCert := "/path/to/cert.pem"
+	ldapKey := "/path/to/key.pem"
+	ldapCAchain := "/path/to/ca_chain.pem"
 
 	// Load client cert and key
 	cert, err := tls.LoadX509KeyPair(ldapCert, ldapKey)
